/**
 * example of plugin usable in jrCMS
 *
 * Conventions:
 *	- the class have to be named with two leading Upper Character (e.g. JRExample)
 *	- the plugin directory must have the same name as the class, but all in lower cases
 *	- each public object (class) gets a four digit ID (id: 4000+)
 *	- the last two digits of the ID are used to identify the methods/functions
 *
 * Methods/functions could be called from the menu if the class and method/function is 
 * announced in the database. The generic parameter for such a call are a root id,
 * menu id and css id, all provided as object (e.g.{root:.., id:..,cssid:..});
 * The initialization of DB (write function to be used) is done by using 'jrCmsRshSource' 
 * below.
 *
 * For traces an internal Logging Function could be used. The logging/debugging function 
 * could be activated via Config.js, option jr_debug with different levels. The logging
 * require an active coding (see below). The first letter is reserved:
 *	- 'T:n#>>' n trace level (1..3)
 *	- 'F#>>' Failure
 *
 * @package Example
 * @author Jens Raabe <jens@raabe-berlin.de>
 * @version $Id: jrcmsexample.js 604 2011-09-17 20:24:32Z jensraabe $ 
 * 
 * @copyright Copyright (c) 2010 Jens Raabe
 * license: MIT-style license, see jrcms.js
 */

/**
 * Example of jrCMS plugin
 * @class
 * @author Jens Raabe
 */

var JRCmsExample = new Class({
	
	_pluginInfo: {
		name:	 		"JRCmsExample",
		version:	 	"0.3.0",					//Release.Version.Minor
		revision:		"$Revision: 604 $",			//Revision set by cvs
		developer:		"Jens Raabe",
		developer_url:	"http://www.raabe-berlin.de",
		license: 		"MIT",
		classId:		1900
	}  
});

(function(){
	JRCmsExample.implement({
		
		Implements: [Options, JRCmsApi]

		//own options
		,options: {
			jr_debug: false,
			jr_lang: 'de'
		}
		
		,lang:{}	//language support
		
		,getClassId: function() {
			return this._pluginInfo.classId;
		}
	
		/*Public*/
		,getVersion: function() {
			return (this._pluginInfo.version+'_'+
					this._pluginInfo.revision.slice(this._pluginInfo.revision.search(/:.+/)+1)).replace(/\$/g,'').replace(/ /g,'');
		}
	
		/**
		 * Constructor: initialize
		 * Creates an instance of the class. 
		 * The contructor will be called during start up if the 
		 * class is announced in the config.js
		 *
		 * Arguments:
		 *  options - (object) The options above.
		 *  instanceName - (string)
		 * Syntax:
		 *  var jrCmsExample = new JRCmsExample(options, instanceName);
		 */
		,initialize: function(options, instanceName){
			
			// must be placed here
			this.setOptions(options);
			this.setOptions({jr_phpSRC: 'jrcmsexample/jrcmsexample.php'});
	
			//optional retrieve language items
			// if(this.writeRights(this.getUserLevel(), this.options.jr_editorLevel))
			this.retrieveLangFile(this.options.jr_pluginsUrl+'jrcmsexample/lang/', 'lang.' + this.options.jr_lang);
	
			//optional retrieve plugin styles
			this.loadStyleFiles({'cssfile':'JRCmsExample.css'
								,'path': this.options.jr_pluginsUrl+'jrcmsexample/css/'});		
			
//			//optional load the editor if needed, but normally this should be done 
//			//depending on the userlevel during startup proceedure, see config
//			if(!$defined(window['jrCmsEditor']))
//				this.loadScriptFiles({'jsfile':'jrcmseditor.js'
//									 ,'path': this.options.jr_pluginsUrl+'jrcmseditor/'
//									 ,'loadfunc': function(){
//										//instantiate the plugin
//										window['jrCmsEditor'] = new JRCmsEditor(this.options, 'jrCmsEditor');
//										this.confEditor();
//									 }.bind(this)});
//			else
//				this.confEditor();
//	
			//register methods external usable (ajax history pool)
			this.registerFunctions({instance: ($defined(instanceName)?instanceName:'jrCmsExample'),
				classId:this.getClassId(),
				functions:[{functionName:'getExample', methodId: 1, functionType: 'ajax' }
							,{functionName:'getHtmlOnly', methodId: 2, functionType: 'ajax' }
				           ]});
	
		}
		
		/**
		 * Refresh constructor
		 * Updates the instance in case of reloading the plugin
		 *
		 * @param array options
		 * @access public
		 */
		,refresh: function(options) {
			; // nothing to do
		}

		/**
		 * get
		 * parameter shall be supplied as json string = object
		 *
		 * @param int $root, int $id, int $cssid
		 * @access public
		 */
		,getExample: function(p, el) {
			
			var methodId = 1; //unique id of method within class
			try{
				//----
				//load the editor if needed, but normally this should be done 
				//depending on the userlevel during startup proceedure, see config
				//we need this for the examples
				if(!$defined(window['jrCmsEditor']))
					this.loadScriptFiles({'jsfile':'jrcmseditor.js'
										 ,'path': this.options.jr_pluginsUrl+'jrcmseditor/'
										 ,'loadfunc': function(){
											//instantiate the plugin
											window['jrCmsEditor'] = new JRCmsEditor(this.options, 'jrCmsEditor');
											this.confEditor();
										 }.bind(this)});
				else
					this.confEditor();
				//----
				
				// action to be used to control the PHP file
				p.a = 'action';
	
				// Ajax history if needed
				var hpParam = this.addRequestHP(this.getClassId()+methodId, p, 'getExample');
												
				// some actions
				// e.g. usage of languages
				// this.lang["s_eventdate"]);
												
				//retrieve and process the content
				this.processRequest({
					requestType: 'xol',
					url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
					data: p,
					_hp: hpParam,
					onSuccess: function(txt, xmltxt, xol){
	
						this.jrLog("T:2#>> wmExample, Request received: \'action\'",2);
	
						// example of own content routine, where the first parameter 
						// contains the innerHTML of the element
						/**@inner*/
						p.contMainFunc = function(content){
							if($defined($(jrCMS.options.jr_divMain))) 
								$(jrCMS.options.jr_divMain).set('html',content);
						};
						// processing of XML container
						this.processXolContainer(xol, p, p);
						
					}.bind(this),
					onFailure: function(err){
						this.jrLog("F#>> jrCmsExample no content "+err);
					}.bind(this),
					onRequest: function(){
						this.jrLog("T:2#>> jrCmsExample, Request send: \'action\'",2);
					}.bind(this),
					onComplete: function(){
						this.jrLog("T:2#>> jrCmsExample, Request completed: \'action\'",2);
					}.bind(this),					
					onCancel: function(){
						this.jrLog("T:2#>> jrCmsExample, Request canceled: \'action\'",2);
					}.bind(this),					
					onException: function(headerName, value){
						this.jrLog("T:2#>> jrCmsExample, Request exception: \'action\' Header: "+headerName+" Value: "+value,2);
					}.bind(this)					
				});
	
			} catch(err){
				this.jrLog("F#>> jrCmsExample.getExample failure: "+err);
			}
		}
			
		/**
		 * get HTML only for divMain
		 * 
		 * @param int $root, int $id, int $cssid
		 * @access public
		 */
		,getHtmlOnly: function (p, el){
			
			try{
				var methodId = 2; //unique id of method within class
	
				// Simulate ajax history for usage of back button
				var hpParam = this.addRequestHP(this.getClassId()+methodId, p, 'getHtmlOnly');
				//set additional parameter
				p.a = 'html';
				p._div = this.options.jr_divMain;
				
				this.processRequest({
					requestType: 'html',
					url: this.options.jr_pluginsUrl + this.options.jr_phpSRC,
					data: p,
					_hp: hpParam
				});
				
			} catch(err){
				this.jrLog("F#>> jrCmsExample.getHtmlOnly failure: "+err);
			}
		}	

		// ////////////////////////////////////////////////////////////////////////////////////////////////////
		// EXAMPLE of DIALOG usage 
		// ////////////////////////////////////////////////////////////////////////////////////////////////////
		
		// **** Example of Dialog Arrays ****
		,myTitles: function() {
			return [{idx:"title-man", value:this.getLangItem('title-man')}
					,{idx:"title-woman",value:this.getLangItem('title-woman')}
					,{idx:"title-man-dr",value:this.getLangItem('title-man-dr')}
					,{idx:"title-woman-dr",value:this.getLangItem('title-woman-dr')}];
		}
		,myCheckboxes: function() {
			return [{idx:"yes", value:this.getLangItem('yes')}
					,{idx:"no",value:this.getLangItem('no')}];
		}
		
		// **** Example Dialog Values ****
		,processDialogValue: function(el, valueName, value){
			switch (valueName) {
			case 'title':
				// === select options ===
				return this.elOptions( el, valueName, value, this.myTitles());			
				break;
			case 'accept':
				// === special input type checkbox ===
				return this.elCheckbox( el, valueName, value, this.myCheckboxes());
				break;
			case 'select':
				// === special input type radiobox ===
				return this.elRadiobox( el, valueName, value, this.myCheckboxes());
				break;
			case 'password':
				// === special input type password ===
				return this.elPassword( el, valueName, value);
				break;
			case 'title_event':
				// === select options, written on event ===
				return this.elOptions( el, valueName, value, this.myTitles.bind(this));			
				break;
			case 'mygrid':
				// === submenu grid ===
				return this.elSubDlg( el, valueName, value, this.myDialogGrid.bind(this));			
				break;
			case 'mycalendar':
				// === submenu calendar ===
				return this.elCalendar( el, valueName, value);			
				break;
			default:
				break;
			}
			return el;
		}
		
		//for processing of elements with validation rules
		,saveDialogValue: function(el) {
			switch (el.get('name')) {
			default:
				break;
			}
			return true;
		}
		
		// **** Example 1) Simple Dialog ****
		// without popup frame
		,testDialog: function() {
			$("divOutput").set('html','');
			//retrieve and process the content
			this.processRequest({
				requestType: 'xol',
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: {a:'testDialog'},
				onSuccess: function(resp, xml, xol){
//return false;  // statement to be used to trace several other data formats, see example.php
					this.writeDialog(
							xol
							,{	'dlgContent': 'divOutput'
								,'dlgDebug': 4	//only necessary to show the output, see option.dlgDebugOutput
								,'dlgText': this.getLangItemSet()
								,'dlgUseFieldset': true
								,'dlgUrl': this.options.jr_pluginsUrl
								,'dlgSrc': this.options.jr_phpSRC
								,'dlgLeftWidth': '100px'
								,'dlgRightWidth': '250px'
					});
				}.bind(this)
			});
		}

		// **** Example 2) Top Dialog ****
		// with popup frame
		,testDialogTop: function() {
			$("divOutput").set('html','');
			//retrieve and process the content
			this.processRequest({
				requestType: 'xol',
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: {a:'testDialogTop'},
				onSuccess: function(resp, xml, xol){
					this.createDialog({
						 'dlgDebug': 4 //only necessary to show the output, see option.dlgDebugOutput
						 ,'dlgText': this.getLangItemSet()
						 ,'dlgTitle': 'myMenuTitle'
						 ,'dlgType': 'top'
				         ,'dlgDragable': true
				         ,'dlgSizable':	true
				         ,'dlgScroller': true
				         ,'dlgMenu': true
						 ,'dlgUseFieldset': true
						 ,'dlgUrl': this.options.jr_pluginsUrl
						 ,'dlgSrc': this.options.jr_phpSRC
						 ,'dlgFrameStyle': 'width:400px;'
						 ,'dlgLeftWidth': '100px'
						 ,'dlgRightWidth': '250px'
						 ,'dlgFadeTime': '500'
						 ,'dlgButtons': {
								'mySave': function(){this.saveDialog({a:'hallo','formOnly':['set2']});}.bind(this)
								}	
						}, xol );
				}.bind(this)
			});
		}

		// **** Example 3) Left Dialog ****
		// with popup frame
		,testDialogLeft: function() {
			$("divOutput").set('html','');
			//retrieve and process the content
			this.processRequest({
				requestType: 'json',
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: {a:'testDialogLeft'},
				onSuccess: function(resp){
					this.createDialog({
						 'dlgDebug': 4  //only necessary to show the output, see option.dlgDebugOutput
						 ,'dlgText': this.getLangItemSet()
						 ,'dlgType': 'Left'
						 ,'dlgUseFieldset': true
						 ,'dlgUrl': this.options.jr_pluginsUrl
						 ,'dlgSrc': this.options.jr_phpSRC
						 ,'dlgFrameStyle': 'width:400px;height:400px'
						 ,'dlgLeftWidth': '100px'
						 ,'dlgRightWidth': '250px'
						 ,'dlgFadeTime': '2500'
						 ,'dlgButtons': {
								'mySave': function(){this.saveDialog({a:'hallo','formOnly':['set4']});}.bind(this)
								}	
						}, resp );
				}.bind(this)
			});
		}
		
		// **** Example 4) Right Dialog ****
		// with popup frame
		,testDialogRight: function() {
			$("divOutput").set('html','');
			this.createDialog({
				 'dlgText': this.getLangItemSet()
				 ,'dlgContent': 'dlg2'
				 ,'dlgType': 'Right'
				 ,'dlgFrameStyle': 'width:400px;height:300px'
				});
			this.processRequest({
				requestType: 'html',
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: {a:'testDialogRight'},
				onSuccess: function(responseTree, responseElements, responseHTML, responseJavaScript){
					$('dlg2').set('html',responseHTML);
				}
			});
		}
		
		// **** Example for separate save function for testDialog
		,testSaveDialog: function(){
			this.saveDialog({a:'hallo','formOnly':['set1', 'set3']});
		}
		
		// **** Example Top Dialog with dyn. Select ****
		// with popup frame
		,testDynDialog: function() {
			$("divOutput").set('html','');
			//retrieve and process the content
			this.processRequest({
				requestType: 'xol',
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: {a:'testDynDialog'},
				onSuccess: function(resp, xml, xol){
					this.createDialog({
						 'dlgDebug': 4 //only necessary to show the output, see option.dlgDebugOutput
						 ,'dlgText': this.getLangItemSet()
						 ,'dlgTitle': 'myMenuTitle'
						 ,'dlgType': 'top'
				         ,'dlgDragable': true
				         ,'dlgSizable':	true
				         ,'dlgScroller': true
				         ,'dlgMenu': true
						 ,'dlgUseFieldset': true
						 ,'dlgUrl': this.options.jr_pluginsUrl
						 ,'dlgSrc': this.options.jr_phpSRC
						 ,'dlgFrameStyle': 'width:400px;'
						 ,'dlgLeftWidth': '100px'
						 ,'dlgRightWidth': '250px'
						 ,'dlgFadeTime': '500'
						 ,'dlgButtons': {
								'mySave': function(){this.saveDialog({a:'hallo','formOnly':['set1']});}.bind(this)
								}	
						}, xol );
				}.bind(this)
			});
		}
		
		// ////////////////////////////////////////////////////////////////////////////////////////////////////
		// EXAMPLE of DIALOG with Editor 
		// ////////////////////////////////////////////////////////////////////////////////////////////////////
		
		// **** Example 5,6 conf editor ****
		,confEditor: function(){
			//wait for domReady in case of fast requests
			if(!$defined(window.jrCmsEditor) || !$type(window.jrCmsEditor) || jrCmsEditor.addEditorAreaEntered) {
				this.confEditor.delay(50,this);
				this.jrLog('T:3#>> jrCmsExample:confEditor wait for domReady! ',3);
				return false;
			}
			this.jrLog('T:3#>> jrCmsExample:confEditor domReady! ',3);
				
			jrCmsEditor.addEditorArea({"remarkXinha":{
				"width":"element",
				"height":"element",
				"statusBar": false,
				"xinaPlugins": false,
				"showLoading": false,
				"toolbar" : [
		             ["bold","italic","underline","strikethrough"],
				     ["separator","insertorderedlist"],
		             ["separator","htmlmode"]],
				    "genSaveButton" : false, 			             
				    "whenXinhaDocReady" : function(){
						;
						//TODO: not necessary because default thumnail is used.							
			    		}.bind(this)        
					}});
			jrCmsEditor.addEditorArea({"accOneXinha":{
				"width":"element",
				"height":"element",
				"statusBar": false,
				"xinaPlugins": false,
				"showLoading": false,
				"toolbar" : [
		             ["bold","italic","underline","strikethrough"],
				     ["separator","insertorderedlist"],
		             ["separator","htmlmode"]],
				    "genSaveButton" : false, 			             
				    "whenXinhaDocReady" : function(){
						;
						//TODO: not necessary because default thumnail is used.							
			    		}.bind(this)        
					}});
			jrCmsEditor.addEditorArea({"accTwoXinha":{
				"width":"element",
				"height":"element",
				"statusBar": false,
				"xinaPlugins": false,
				"showLoading": false,
				"toolbar" : [
		             ["bold","italic","underline","strikethrough"],
				     ["separator","insertorderedlist"],
		             ["separator","htmlmode"]],
				    "genSaveButton" : false, 			             
				    "whenXinhaDocReady" : function(){
						;
						//TODO: not necessary because default thumnail is used.							
			    		}.bind(this)        
					}});
		}
		
		// **** Example 5) Dialog with Editor ****
		,testDialogEditor: function() {
			$("divOutput").set('html','');
			//retrieve and process the content
			this.processRequest({
				requestType: 'xol',
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: {a:'testDialogEditor'},
				onSuccess: function(resp, xml, xol){
					this.writeDialog(
							xol
							,{	'dlgContent': 'divOutput'
								,'dlgDebug': 4	//only necessary to show the output, see option.dlgDebugOutput
								,'dlgText': this.getLangItemSet()
								,'dlgUseFieldset': true
								,'dlgUrl': this.options.jr_pluginsUrl
								,'dlgSrc': this.options.jr_phpSRC
								,'dlgLeftWidth': '100px'
								,'dlgRightWidth': '250px'
					});
				}.bind(this)
			});
		}
		
		// **** Example 6) Dialog with Editor ****
		,testDialogAccordionEditor: function() {
			$("divOutput").set('html','');
			//retrieve and process the content
			this.processRequest({
				requestType: 'xol',
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: {a:'testDialogAccordionEditor'},
				onSuccess: function(resp, xml, xol){
					this.writeDialog(
							xol
							,{	'dlgContent': 'divOutput'
								,'dlgDebug': 4	//only necessary to show the output, see option.dlgDebugOutput
								,'dlgText': this.getLangItemSet()
								,'dlgUseFieldset': true
								,'dlgUrl': this.options.jr_pluginsUrl
								,'dlgSrc': this.options.jr_phpSRC
								,'dlgLeftWidth': '100px'
								,'dlgRightWidth': '250px'
					});
				}.bind(this)
			});
		}

		// ////////////////////////////////////////////////////////////////////////////////////////////////////
		// EXAMPLE of DIALOG with BUTTONS (processXolContainer)
		// ////////////////////////////////////////////////////////////////////////////////////////////////////
	
		// **** Example 6) Dialog with Editor ****
		,testDialogWithContentAndButtons: function() {
			
			var opt = { 'dlgContent' : 'myDialog'
					   ,'dlgUseFieldset' : false			//instead of true/false
					   ,'dlgUrl': this.options.jr_pluginsUrl
					   ,'dlgSrc': this.options.jr_phpSRC
					   ,'dlgLeftWidth' : '100px'
					   ,'dlgRightWidth' : '250px'
					  };
			var p = {a:'testDialogWithContentAndButtons',dlgopt:opt};

			$(this.options.jr_divMain).empty();
			// Ajax history if needed
			var hpParam = this.addRequestHP(this.getClassId()+0, p, 'testDialogWithContentAndButtons');
											
			//retrieve and process the content
			this.processRequest({
				requestType: 'xol',
				_hp: hpParam,
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: p
			});
		}
		
		,buttonFunction: function(){
			alert('any functionality');
		}

		// ////////////////////////////////////////////////////////////////////////////////////////////////////
		// EXAMPLE of DIALOG with Subdialogs
		// ////////////////////////////////////////////////////////////////////////////////////////////////////
	
		// **** Example 8) Top Dialog and Grid****
		// with popup frame
		,testDialogTopSubDlg: function() {
			$("divOutput").set('html','');
			//retrieve and process the content
			this.processRequest({
				requestType: 'xol',
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: {a:'testDialogTopSubDlg'},
				onSuccess: function(resp, xml, xol){
					this.createDialog({
						 'dlgDebug': 4 //only necessary to show the output, see option.dlgDebugOutput
						 ,'dlgText': this.getLangItemSet()
						 ,'dlgTitle': 'myMenuTitle'
						 ,'dlgType': 'top'
				         ,'dlgDragable': true
				         ,'dlgSizable':	true
				         ,'dlgScroller': true
				         ,'dlgMenu': true
						 ,'dlgUseFieldset': true
						 ,'dlgUrl': this.options.jr_pluginsUrl
						 ,'dlgSrc': this.options.jr_phpSRC
						 ,'dlgFrameStyle': 'width:600px'
						 ,'dlgLeftWidth': '100px'
						 ,'dlgRightWidth': '450px'
						 ,'dlgFadeTime': '500'
						 ,'dlgButtons': {
								'mySave': function(){this.saveDialog({a:'hallo','formOnly':['set2']});}.bind(this)
								}	
						}, xol );
				}.bind(this)
			});
		}
		
		,myDialogGrid: function(p) {
			
			//define grid options
			var myDialogGridOptions = { 
					columnModel: [{
			       		header: "id"
			       		,dataIndex: 'uid'
			       		,dataType:'number'
			       		,width: 50
			       		}
						,{
						header: "User Name"
						,dataIndex: 'UserName'
						,dataType:'string'
						,width: 120
						}
						,{
						header: "User Level"
						,dataIndex: 'description'
						,dataType:'string'
						,width: 350
						}],
					url: null,
					autoSectionToggle: false,
					perPageOptions: [10],
					perPage: 10,
					page: 1,
					pagination: true,
					serverSort: false,
					showHeader: true,
					sortHeader: false,
					alternaterows: true,
					resizeColumns: false,
					multipleSelection: true,
					//additional parameters
					pluginWidth: null,
					pluginHeight: null,
					pluginFilter: null,
					pluginParam: null
			};

			p.a = 'getUserLst';
			//set further parameter
			myDialogGridOptions.pluginWidth = '450px';
			myDialogGridOptions.pluginHeight = '250px';
			myDialogGridOptions.pluginFilter = true;
			myDialogGridOptions.jr_divGrid = p.el.id;
			myDialogGridOptions.pluginParam = p;
			myDialogGridOptions.url = this.options.jr_pluginsUrl+'jrcmsuser/jrcmsuser.php';
			myDialogGridOptions.accordion = false;
			this.setOptions(myDialogGridOptions);
			this._myDialogGrid = null; 
			this._myDialogGrid = jrCmsGrid.initOmniGrid(this.options.jr_divGrid, this.options);
			this._myDialogGrid.loadData(this.options.url);
//			// fireEvent "click" is coded in omnigrid.onRowClick where
//			// first the internal function is processed and at last step
//			// the event "click" is generated
//			this._myDialogGrid.addEvent('click', this.onGridSelect.bind(this));
//				the onGridSelect have to ensure that the row selected
//				is transfered by the save task.
//				e.g. the ID must be put in a hidden input element.
		}
	
		,_myDialogGrid: null
		,_myDialogGridSelection: null
		,_myDialogGridParam: null
		
		// ////////////////////////////////////////////////////////////////////////////////////////////////////
		// NEW MENU FUNCTIONALITY
		// ////////////////////////////////////////////////////////////////////////////////////////////////////
		
		,testMenu: function() {
			
			var p = {a:'testMenu'};
			this.processRequest({
				requestType: 'xol',
				url: this.options.jr_pluginsUrl+this.options.jr_phpSRC,
				data: p
			});
		}
		
	}); //implements end
})();